package org.gcube.portlets.user.geoportaldataentry;

import java.util.LinkedHashMap;
import java.util.List;

import org.gcube.application.geoportal.common.model.document.Project;
import org.gcube.application.geoportal.common.model.useCaseDescriptor.HandlerDeclaration;
import org.gcube.application.geoportal.common.model.useCaseDescriptor.UseCaseDescriptor;
import org.gcube.application.geoportalcommon.ConvertToDataValueObjectModel;
import org.gcube.application.geoportalcommon.ProjectDVBuilder;
import org.gcube.application.geoportalcommon.geoportal.GeoportalClientCaller;
import org.gcube.application.geoportalcommon.geoportal.ProjectsCaller;
import org.gcube.application.geoportalcommon.geoportal.UseCaseDescriptorCaller;
import org.gcube.application.geoportalcommon.shared.geoportal.ConfigurationDV;
import org.gcube.application.geoportalcommon.shared.geoportal.config.GcubeProfileDV;
import org.gcube.application.geoportalcommon.shared.geoportal.project.ProjectDV;
import org.gcube.application.geoportalcommon.shared.geoportal.ucd.GEOPORTAL_CONFIGURATION_TYPE;
import org.gcube.application.geoportalcommon.shared.geoportal.ucd.GEOPORTAL_DATA_HANDLER;
import org.gcube.application.geoportalcommon.shared.geoportal.ucd.HandlerDeclarationDV;
import org.gcube.common.authorization.library.provider.SecurityTokenProvider;
import org.gcube.common.scope.api.ScopeProvider;
import org.gcube.portlets.widgets.mpformbuilder.server.MetadataProfileFormBuilderServiceImpl;
import org.gcube.portlets.widgets.mpformbuilder.shared.metadata.MetaDataProfileBean;
import org.gcube.portlets.widgets.mpformbuilder.shared.metadata.MetadataFieldWrapper;
import org.junit.Before;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jayway.jsonpath.JsonPath;

public class Complex_Tests {

	private UseCaseDescriptorCaller clientUCD = null;
	private ProjectsCaller clientPrj = null;
	private static String CONTEXT = "/gcube/devsec/devVRE";
	private static String TOKEN = ""; // devVRE
	private static String PROFILE_ID = "profiledConcessioni";

	private static String PROJECT_ID = "630f905855e2947b0278c1a8";
	
	public static final String JSON_$_POINTER = "$";
	private static final Logger LOG = LoggerFactory.getLogger(Complex_Tests.class);

	//@Before
	public void init() {
		ScopeProvider.instance.set(CONTEXT);
		SecurityTokenProvider.instance.set(TOKEN);
		clientPrj = GeoportalClientCaller.projects();
		clientUCD = GeoportalClientCaller.useCaseDescriptors();
	}

	//@Test
	public void testReadProjectForUCDDataEntry() {
		ScopeProvider.instance.set(CONTEXT);
		SecurityTokenProvider.instance.set(TOKEN);

		try {
			System.out.println("Loading UCD for PROFILE_ID: "+PROFILE_ID);
			UseCaseDescriptor ucd = clientUCD.getUCDForId(PROFILE_ID);
			GEOPORTAL_DATA_HANDLER theHandler = GEOPORTAL_DATA_HANDLER.geoportal_data_entry;
			List<HandlerDeclaration> handlers = ucd.getHandlersByType(theHandler.getType());

			if (handlers.size() == 0)
				throw new Exception("No handler " + theHandler + "found");

			//Loading Handler gcube_profiles
			HandlerDeclaration dataEntryHandler = handlers.get(0);
			HandlerDeclarationDV handlerGcubeProfiles = ConvertToDataValueObjectModel
					.toHandlerDeclarationDV(dataEntryHandler, ucd, GEOPORTAL_CONFIGURATION_TYPE.gcube_profiles);
			
			System.out.println("Handler "+GEOPORTAL_CONFIGURATION_TYPE.gcube_profiles+" for PROFILE_ID: "+PROFILE_ID);
			System.out.println(handlerGcubeProfiles);

			ConfigurationDV<?> config = handlerGcubeProfiles.getConfiguration();
			List<GcubeProfileDV> listGcubeProfiles = toListGcubeProfiles(config);
			System.out.println("List of GcubeProfileDV are: "+listGcubeProfiles);
			
			//Loading Metadata Profile from IS
			MetadataProfileFormBuilderServiceImpl metaProfileBUilder = new MetadataProfileFormBuilderServiceImpl();
			LinkedHashMap<String, List<MetaDataProfileBean>> linkedHashProfiles = new LinkedHashMap<String, List<MetaDataProfileBean>>();
			for (GcubeProfileDV gcubeProfileDV : listGcubeProfiles) {
				ScopeProvider.instance.set(CONTEXT);
				SecurityTokenProvider.instance.set(TOKEN);
				List<MetaDataProfileBean> listProfiles = metaProfileBUilder.getProfilesInTheScopeForName(CONTEXT,
						gcubeProfileDV.getGcubeSecondaryType(), gcubeProfileDV.getGcubeName());
				String key = gcubeProfileDV.getGcubeSecondaryType()+gcubeProfileDV.getGcubeName();
				System.out.println("for key: "+key+ " readi profiles: "+listGcubeProfiles);
				linkedHashProfiles.put(gcubeProfileDV.getGcubeSecondaryType()+gcubeProfileDV.getGcubeName(), listProfiles);
			}
			
			Project theProject = clientPrj.getProjectByID(PROFILE_ID, PROJECT_ID);
			ProjectDVBuilder projectBuilder = ProjectDVBuilder.newBuilder().fullDocumentMap(true);
			ProjectDV theProjectDV = ConvertToDataValueObjectModel.toProjectDV(theProject, projectBuilder);
			System.out.println("theProjectDV as JSON: "+theProject.getTheDocument().toJson());
			System.out.println("theProjectDV as MAP: "+theProjectDV.getTheDocument().getDocumentAsMap());
			
			for (GcubeProfileDV gcubeProfileDV : listGcubeProfiles) {
				
				LOG.debug("The profile is: " + gcubeProfileDV);

				// Building JSON/section full PATH and section name
				String sectionJSONPath = "";
				String parentPathFromProfile = gcubeProfileDV.getParentName() == null ? "" : gcubeProfileDV.getParentName();
				String theSectionName = gcubeProfileDV.getSectionName();

				if (theSectionName.compareTo(JSON_$_POINTER) == 0 || theSectionName.compareTo(JSON_$_POINTER + ".") == 0) {
					sectionJSONPath = JSON_$_POINTER;
					theSectionName = "";
				} else {
					sectionJSONPath = String.format("%s%s",
							parentPathFromProfile.endsWith(".") ? parentPathFromProfile : parentPathFromProfile + ".", theSectionName);
				}
				LOG.debug("The sectionJSONPath is: " + sectionJSONPath);

				// Building Parent PATH
				String parentPath = sectionJSONPath.compareTo(JSON_$_POINTER) == 0 ? JSON_$_POINTER
						: sectionJSONPath.substring(0, sectionJSONPath.lastIndexOf("."));
				
				JsonPath parentJSONPath = JsonPath.compile(parentPath);
				
				List<MetaDataProfileBean> theProfileBeans = linkedHashProfiles.get(gcubeProfileDV.getGcubeSecondaryType()+gcubeProfileDV.getGcubeName());
				MetaDataProfileBean theProfileBean = theProfileBeans.get(0);
				for (MetadataFieldWrapper metadataField : theProfileBean.getMetadataFields()) {
					String theFieldName = metadataField.getFieldId()!=null?metadataField.getFieldId():metadataField.getFieldName();
					theProjectDV.getTheDocument().getDocumentAsMap().get(linkedHashProfiles);
				}
				
			}

			for (String key : linkedHashProfiles.keySet()) {
				System.out.println("For key: "+key+" got profiles: "+linkedHashProfiles.get(key));

			}
		} catch (Exception e) {
			System.out.println("error");
			e.printStackTrace();
		}
		
		System.out.println("termimated");
	}

	/**
	 * To list gcube profiles.
	 *
	 * @param config the config
	 * @return the list
	 */
	private List<GcubeProfileDV> toListGcubeProfiles(ConfigurationDV<?> config) {

		try {
			return (List<GcubeProfileDV>) config.getConfiguration();
		} catch (Exception e) {
			System.err.println("Error on casting " + ConfigurationDV.class.getName() + " to List of "
					+ GcubeProfileDV.class.getName());
		}

		return null;
	}

}
